﻿using apifon_sdk_csharp.com.apifon.sdk.helpers;
using apifon_sdk_csharp.com.apifon.sdk.model;
using apifon_sdk_csharp.com.apifon.sdk.model.request;
using apifon_sdk_csharp.com.apifon.sdk.resource;
using apifon_sdk_csharp.com.apifon.sdk.resource.helpers;
using apifon_sdk_csharp.com.apifon.sdk.response;
using com.apifon.sdk;
using com.apifon.sdk.helpers;
using com.apifon.sdk.model.request;
using com.apifon.sdk.resource;
using Newtonsoft.Json;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace apifon_sdk_csharp.com.apifon.Examples
{
    class ListExample
    {
        static void Main(string[] args)
        {
            Mookee.AddCredentials("token", "Your_Token", "Your_Secret_Key");
            Mookee.SetActiveCredential("token");

            ListResource listResource = new ListResource();
            SubscriberFieldResource subscriberFieldResource = new SubscriberFieldResource();
            GdprFieldResource gdprFieldResource = new GdprFieldResource();
            GroupResource groupResource = new GroupResource();
            SubgroupResource subgroupResource = new SubgroupResource();

            int choice = 1;

            //DUMMY uri for Initialization
            Uri uri = new Uri("http://ars.apifon.com/globaluri");
            Uri groupUri = new Uri("http://ars.apifon.com/services/api/v1/list{list_id}/group/{group_field}");
            Uri subgroupUri = new Uri("http://ars.apifon.com/services/api/v1/list/{list_id}/{group_field}/option/{option_field}}");

            string listId = "";
            string groupField = "";
            string optionsField = "";

            while (choice != 0)
            {
                Console.WriteLine("------Menu------");
                Console.WriteLine("0.  Exit");
                Console.WriteLine("1.  Create a Simple List");
                Console.WriteLine("2.  Create a Complete List");
                Console.WriteLine("3.  Get a List with a Uri");
                Console.WriteLine("4.  Get a List with listId");
                Console.WriteLine("5.  Get All Lists ");
                Console.WriteLine("6.  Update a List");
                Console.WriteLine("7.  Remove a List");
                Console.WriteLine("8.  Add a Subscriber Field to List");
                Console.WriteLine("9.  Update a Subscriber Field to List");
                Console.WriteLine("10.  Remove a Subscriber Field from List");
                Console.WriteLine("11.  Add a GDPR Field to List");
                Console.WriteLine("12.  Update a GDPR Field to List");
                Console.WriteLine("13.  Add Group");
                Console.WriteLine("14.  Get Group with uri");
                Console.WriteLine("15.  Get Group with group field");      
                Console.WriteLine("16.  Update Group");
                Console.WriteLine("17.  Remove Group");
                Console.WriteLine("18.  Add Subgroup to Group");
                Console.WriteLine("19.  Add Subgroups to Group");
                Console.WriteLine("20.  Get Subgroup by uri");
                Console.WriteLine("21.  Get Subgroup by options field");
                Console.WriteLine("22.  Update Subgroup");
                Console.WriteLine("23.  Remove Subgroup");
                Console.Write("Choice: ");
                choice = Convert.ToInt32(Console.ReadLine());

                switch (choice)
                {
                    case 0:
                        break;

                    /**
                     * Create a Simple List
                     * 
                     * Use ListType enum class in order to send an acceptable string to API.
                     */
                    case 1:
                        ListRequestBuilder.ListRequest listRequest = ListRequestBuilder.getAddListRequestBuilder(ListType.ADVANCED.ToString(), "List from C#", "+30")
                            .build();
                        Console.WriteLine("Sending Request to Endpoint: /services/list/");
                        Console.WriteLine("------ List Request Body ------");
                        PrintJson(listRequest);
                        uri = listResource.createList(listRequest);
                        Console.WriteLine(uri);

                        break;

                    /**
                    * Create a Complete List with Campaign Defaults and GDPR Properties
                    * 
                    * Use ListType enum class in order to send an acceptable string to API.
                    */
                    case 2:
                        CampaignDefaultsBuilder campaignDefaultsBuilder = CampaignDefaultsBuilder.getCampaignDefaultsBuilder();
                        campaignDefaultsBuilder.setSenderId("Apifon").setSenderName("Apifon").setViberSenderId("Apifon").setFrom("apifon@apifon.com").setReplyTo("apifon@apifon.com");

                        GdprPropertiesBuilder gdprPropertiesBuilder = GdprPropertiesBuilder.getGdprPropertiesBuilder(true);
                        gdprPropertiesBuilder.setTitle("GDPR title").setLegalText("GDPR legal text").setDescription("GDPR description");

                        ListRequestBuilder.ListRequest listRequest2 = ListRequestBuilder.getAddListRequestBuilder(ListType.ADVANCED.ToString(), "List from C#", "+30")
                            .withCampaignDefaults(campaignDefaultsBuilder)
                            .withGdprProperties(gdprPropertiesBuilder)
                            .build();
                        Console.WriteLine("Sending Request to Endpoint: /services/list/");
                        Console.WriteLine("------ List Request Body ------");
                        PrintJson(listRequest2);
                        uri = listResource.createList(listRequest2);
                        Console.WriteLine(uri);

                        break;

                    /**
                    * Get a List with a Uri
                    */
                    case 3:
                        ListResponse list = listResource.getByResourceUri(uri);
                        listId = list.getId();
                        PrintJson(list);

                        break;

                    /**
                    * Get a List with listId (*optional: You can add fields for partial response)
                    * 
                    * You need to get a ListResponse first --> case 3
                    * 
                    * in order to have the list_id
                   */
                    case 4:
                        ListSingleQuery qry = ListSingleQuery.getListRequest(listId);
                        Collection<string> fields = new Collection<string>();

                        //Fields for partial response
                        fields.Add("id");
                        fields.Add("title");
                        qry.addVisibleFields(fields);

                        ListResponse list2 = listResource.getList(qry);
                        PrintJson(list2);

                        break;

                    /**
                    * Get All Lists by using ListQueryBuilder (*optional: You can add fields and query params)
                    */
                    case 5:
                        Collection<string> fields2 = new Collection<string>();

                        PaginatedRequest paginatedRequest = ListQueryBuilder.getBuilder()
                            .setSort("title", SortDirection.ASC)
                            .build();
                        CollectionEntityResponse<ListResponse> collection = listResource.getLists(paginatedRequest);
                        PrintJson(collection);

                        break;

                    /**
                    * Update a List
                    * 
                    * You need to get a ListResponse first --> case 3
                    * 
                    * in order to have the list_id
                    */
                    case 6:
                        //GDPR Properties NEEDED!
                        GdprPropertiesBuilder gdprPropertiesBuilder2 = GdprPropertiesBuilder.getGdprPropertiesBuilder(false);

                        ListRequestBuilder listRequestBuilder = ListRequestBuilder.getUpdateListRequestBuilder(listId);
                        ListRequestBuilder.ListRequest listRequest1 = listRequestBuilder.withTitle("Updated title")
                            .withPrefix("+30")
                            .withGdprProperties(gdprPropertiesBuilder2) //GDPR Properties NEEDED!
                            .build();
                        listResource.updateList(listRequest1);

                        break;

                    /**
                    * Remove a List
                    * 
                    * You need to get a ListResponse first --> case 3
                    * 
                    * in order to have the list_id
                    */
                    case 7:
                        listResource.removeList(listId);

                        break;

                    /**
                    * Add a Subscriber Field to List
                    * 
                    * You need to get a ListResponse first --> case 3
                    * 
                    * in order to have the list_id
                    * 
                    * Use SubscriberFieldType and SubscriberFieldDateFormat enum classes in order to send acceptable strings to API.
                    * 
                    * SubscriberFieldDateFormat.dd_mm_yyyy.Value --> *** Value ***, not ToString()
                    */
                    case 8:
                        SubscriberFieldRequestBuilder.SubscriberFieldRequest subscriberFieldRequest = SubscriberFieldRequestBuilder
                            .getAddRequestBuilder(listId, SubscriberFieldType.DATE.ToString(), 20, "Birthday")
                            .withDateFormat(SubscriberFieldDateFormat.dd_mm_yyyy.Value) // Date format is Obligatory when SubscriberFieldType is DATE
                            .build();
                        Uri subscriberFieldUri = subscriberFieldResource.addSubscriberField(subscriberFieldRequest);
                        Console.WriteLine(subscriberFieldUri);

                        break;

                    /**
                    * Update a Subscriber Field to List
                    * 
                    * You need to get a ListResponse first --> case 3
                    * 
                    * in order to have the list_id
                    */
                    case 9:
                        SubscriberFieldRequestBuilder.SubscriberFieldRequest subscriberFieldRequest1 = SubscriberFieldRequestBuilder
                            .getUpdateSubscriberFieldRequestBuilder(listId, "date_BIRTHDAY", "Birthday", 15)
                            .build();
                        subscriberFieldResource.updateSubscriberField(subscriberFieldRequest1);

                        break;

                    /**
                    * Remove a Subscriber Field from List
                    * 
                    * You need to get a ListResponse first --> case 3
                    * 
                    * in order to have the list_id
                    */
                    case 10:
                        subscriberFieldResource.removeSubscriberField(listId, "date_BIRTHDAY");

                        break;

                    /**
                    * Add a GDPR Field to List
                    * 
                    * You need to get a ListResponse first --> case 3
                    * 
                    * in order to have the list_id
                    * 
                    * *** List MUST be GDPR compliant
                    */
                    case 11:
                        GdprFieldRequestBuilder.GdprFieldRequest gdprFieldRequest = GdprFieldRequestBuilder
                            .getAddGdprFieldRequestBuilder("First name", listId).build();
                        Uri gdprFieldUri = gdprFieldResource.addGdprField(gdprFieldRequest);
                        Console.WriteLine(gdprFieldUri);

                        break;

                    /**
                    * Update a GDPR Field to List
                    * 
                    * You need to get a ListResponse first --> case 3
                    * 
                    * in order to have the list_id
                    * 
                    * *** List MUST be GDPR compliant
                    */
                    case 12:
                        GdprFieldRequestBuilder.GdprFieldRequest reqUpd = GdprFieldRequestBuilder
                            .getUpdateGdprFieldRequestBuilder("First Name", listId, "FIRST_NAME").withName("Updated")
                            .build();
                        gdprFieldResource.updateGdprField(reqUpd);

                        break;

                    /**
                    * Add a group to List
                    * 
                    * You need to get a ListResponse first --> case 3
                    * 
                    * in order to have the list_id
                    *
                    */
                    case 13:
                        GroupRequestBuilder.GroupRequest addGroupRequest = GroupRequestBuilder.getAddGroupRequestBuilder(listId, GroupType.GROUP.ToString(), "my c# group")
                            .withSelectionType(SelectionType.MULTIPLE.ToString()).build();

                        groupUri = groupResource.addGroup(addGroupRequest);
                        break;
                    /**
                    * Get a Group with a Uri
                    */
                    case 14:

                        GroupResponse myGroupFromUri = groupResource.getByResourceUri(groupUri);
                        groupField = myGroupFromUri.getField();
                        PrintJson(myGroupFromUri);
                        break;


                    /**
                      * Get a group from List
                      * 
                      * You need to get a ListResponse first --> case 3
                      * 
                      * in order to have the list_id and groupField
                      * 
                      * *** List MUST contain a group already
                      */
                    case 15:

                        GroupSingleQuery getGroupRequest = GroupSingleQuery.getGroupRequest(listId, groupField);

                        GroupResponse myGroup = groupResource.getGroup(getGroupRequest);
                        PrintJson(myGroup);
                        break;

                
                    /**
                    * Update a group
                    * 
                    * You need to get a ListResponse first --> case 3
                    * 
                    * in order to have the list_id
                    *
                    * You could also get a Group Response first and pass it in the getUpdateGroupRequestBuilder() method  --> case 14
                    *
                    *
                    */
                    case 16:
                        GroupRequestBuilder.GroupRequest updateGroupRequest = GroupRequestBuilder.getUpdateGroupRequestBuilder(listId, groupField, GroupType.GROUP.ToString(), "updated group name 2").build();

                        groupResource.updateGroup(updateGroupRequest);
                        break;
                    /**
                       *
                       * Remove a group from a list with the group field
                       *
                       *** WARNING this operation is irreverible!
                       */
                    case 17:

                        groupResource.removeGroup(listId, groupField);
                        break;
                    /**
                     * Add a subgroup to a group
                     * 
                     * You need to get a ListResponse first --> case 3
                     * 
                     * in order to have the list_id and the group field
                     * 
                     * 
                     *
                     */
                    case 18:
                        SubgroupRequestBuilder.SubgroupRequest addSubgroupRequest = SubgroupRequestBuilder.getAddSubgroupRequestBuilder(listId, groupField, "my option").build();

                        subgroupResource.addSubgroup(addSubgroupRequest);
                        break;
                     
                    /**
                     * Add a subgroups to a group
                     * 
                     * You need to get a ListResponse first --> case 3
                     * 
                     * in order to have the list_id and the group field
                     * 
                     * 
                     *
                     */
                    case 19:
                        List<String> myOptions = new List<String>();
                        myOptions.Add("option1");
                        myOptions.Add("option2");
                        SubgroupRequestBuilder.SubgroupRequest addSubgroupsRequest = SubgroupRequestBuilder.getAddSubgroupsRequestBuilder(listId, groupField, myOptions).build();

                        subgroupUri = subgroupResource.addSubgroups(addSubgroupsRequest);
                        break;
                    /**
                    * Get subgroup by uri
                    * 
                    */
                    case 20:

                        GroupResponse mySubgroupFromUri = groupResource.getByResourceUri(subgroupUri);
                        optionsField = mySubgroupFromUri.getField();
                        PrintJson(mySubgroupFromUri);
                        break;
                    /**
                      * Get a subgroup from Group
                      * 
                      * You need to get a ListResponse first --> case 3
                      * 
                      * in order to have the list_id and groupField
                      * 
                      * *** List MUST contain a group already
                      */
                    case 21:

                        SubgroupSingleQuery subgroupSingleQuery = SubgroupSingleQuery.getGroupRequest(listId, groupField, optionsField);

                        SubgroupResponse mySubgroup = subgroupResource.getSubgroup(subgroupSingleQuery);
                        PrintJson(mySubgroup);
                        break;

                    /**
                    * Add a subgroup to a group
                    * 
                    * You need to get a ListResponse first --> case 3
                    * 
                    * in order to have the list_id and the group field
                    * 
                    * 
                    *
                    */
                    case 22:
                        SubgroupRequestBuilder.SubgroupRequest updateSubgroupRequest = SubgroupRequestBuilder.getUpdateSubgroupRequestBuilder(listId, groupField, optionsField, "updated option name").build();

                        subgroupResource.updateSubgroup(updateSubgroupRequest);
                        break;
                    /**
                    * remove subgroup
                    */
                    case 23:
                        subgroupResource.removeSubgroup(listId, groupField, optionsField);
                        break;


                }


            }



        }

        public static void PrintJson(Object response)
        {
            String jsonString = JsonConvert.SerializeObject(response);
            Console.WriteLine(JsonHelper.FormatJson(jsonString));
        }
    }
}


