﻿using apifon_sdk_csharp.com.apifon.sdk.helpers;
using apifon_sdk_csharp.com.apifon.sdk.model;
using apifon_sdk_csharp.com.apifon.sdk.model.request;
using apifon_sdk_csharp.com.apifon.sdk.resource;
using apifon_sdk_csharp.com.apifon.sdk.resource.helpers;
using apifon_sdk_csharp.com.apifon.sdk.response;
using com.apifon.sdk;
using com.apifon.sdk.helpers;
using com.apifon.sdk.model;
using com.apifon.sdk.model.request;
using com.apifon.sdk.resource;
using com.apifon.sdk.response;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using static apifon_sdk_csharp.com.apifon.sdk.model.request.OAuthConfigurationBuilder;

namespace apifon_sdk_csharp.com.apifon.Examples
{
    public class OauthExample
    {
        private static readonly String OAUTH_IDENTIFIER = "YOUR_IDENTIFIER";
        private static readonly String CLIENT_ID = "YOUR_CLIENT_ID";
        private static readonly String CLIENT_SECRET = "YOUR_CLIENT_SECRET";

        // Using a ListResource to call list-related APIs
        private static readonly ListResource LIST_RESOURCE = new ListResource();

        static void Main(string[] args)
        {
            clientCredentialsGrantExample();
        }

        private static void clientCredentialsGrantExample() 
        {
            // Create an OAuth configuration to use
            OAuthConfiguration oauthConfig = getClientCredentialsGrantConfiguration(
                    CLIENT_ID,
                    CLIENT_SECRET
            );

            // Setting the OAuth configuration to Mookee instance!
            Mookee.addOAuthResource(OAUTH_IDENTIFIER, oauthConfig);
            // Retrieve an OAuth2 token from the Identity server using the configuration.
            Mookee.retrieveClientCredentialsToken(OAUTH_IDENTIFIER);
            // Setting the Mookee to OAuth mode so that OAuth is used for any subsequent API calls.
            Mookee.setMookeeToOAuthMode(OAUTH_IDENTIFIER, true);
            // Call API to create List
            ListResponse createListResponse = createList();
            Console.WriteLine("Created list with ID : {0} " + createListResponse.getId());
            // Revoke active tokens!
            Mookee.removeOAuthResource(OAUTH_IDENTIFIER, true);

        }

        private static ListResponse createList()
        {
            ListRequestBuilder.ListRequest request = ListRequestBuilder.getAddListRequestBuilder(ListType.ADVANCED.ToString(), "OAuth2 List", "30")
                .withDescription("This is a list created using the SDK and an OAuth2 Bearer token!")
                .withCampaignDefaults(
                        CampaignDefaultsBuilder.getCampaignDefaultsBuilder()
                                .setFrom("email@apifon.com")
                                .setReplyTo("noreply@apifon.com")
                                .setSenderName("Apifon")
                                .setSenderId("Apifon")
                                .build()
                )
                .withGdprProperties(
                        GdprPropertiesBuilder.getGdprPropertiesBuilder(true)
                                .setLegalText("My Legal Text")
                                .setTitle("Communication Purposes")
                                .setDescription("Information about the purposes we wish to contact you.")
                )
                .build();
            Uri createdListResource = LIST_RESOURCE.createList(request);
            return LIST_RESOURCE.getByResourceUri(createdListResource);
        }

        private static OAuthConfiguration getClientCredentialsGrantConfiguration(string clientId, string clientSecret)
        {
            return OAuthConfigurationBuilder
                    .getClientCredentialsBuilder(clientId, clientSecret)
                    .build();
        }
    }   
}

