﻿using apifon_sdk_csharp.com.apifon.sdk.helpers;
using apifon_sdk_csharp.com.apifon.sdk.response;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using System.Web.Script.Serialization;

namespace apifon_sdk_csharp.com.apifon.sdk.model.request
{
    class GroupRequestBuilder
    {
        private readonly string listId;
        private readonly string groupField;
        private string name;
        private string type;
        private string selectionType;
        private string groupOptionsOrientation;
        private bool internal_;

        private GroupRequestBuilder(string listId, string type, string name, string groupField)
        {
            this.listId = listId;
            this.type = type;
            this.name = name;
            this.groupField = groupField;
        }

        public static GroupRequestBuilder getAddGroupRequestBuilder(string listId, string type, string name)
        {
            if (string.IsNullOrEmpty(listId)) { throw new System.ArgumentNullException("Parameter listId is null."); }
            if (string.IsNullOrEmpty(name)) { throw new System.ArgumentNullException("Parameter name is null."); }
            if (string.IsNullOrEmpty(type)) { throw new System.ArgumentNullException("Parameter type is null."); }
            return new GroupRequestBuilder(listId, type, name, null);
        }

        public static GroupRequestBuilder getUpdateGroupRequestBuilder(string listId, string groupField, string type, string name)
        {
            if (string.IsNullOrEmpty(listId)) { throw new System.ArgumentNullException("Parameter listId is null."); }
            if (string.IsNullOrEmpty(name)) { throw new System.ArgumentNullException("Parameter name is null."); }
            if (string.IsNullOrEmpty(type)) { throw new System.ArgumentNullException("Parameter type is null."); }
            if (string.IsNullOrEmpty(groupField)) { throw new System.ArgumentNullException("Parameter groupField is null."); }

            return new GroupRequestBuilder(listId, type, name, groupField);
        }

        public static GroupRequestBuilder getUpdateGroupRequestBuilder(GroupResponse groupResponse, String listId)
        {
            if (string.IsNullOrEmpty(listId)) { throw new System.ArgumentNullException("Parameter listId is null."); }
            if (groupResponse == null) { throw new System.ArgumentNullException("Parameter groupResponse is null."); }
            return new GroupRequestBuilder(listId, groupResponse.getType(), groupResponse.getName(), groupResponse.getField());
        }

        public GroupRequestBuilder withName(string name)
        {
            if (string.IsNullOrEmpty(name)) { throw new System.ArgumentNullException("Parameter name is null."); }
            this.name = name;
            return this;
        }


        public GroupRequestBuilder withType(string type)
        {
            if (string.IsNullOrEmpty(type)) { throw new System.ArgumentNullException("Parameter type is null."); }
            this.type = type;
            return this;
        }

        public GroupRequestBuilder withInternal(bool internal_)
        {
            this.internal_ = internal_;
            return this;
        }


        public GroupRequestBuilder withSelectionType(string selectionType)
        {
            if (string.IsNullOrEmpty(selectionType)) { throw new System.ArgumentNullException("Parameter selectionType is null."); }
            this.selectionType = selectionType;
            return this;
        }

        public GroupRequestBuilder withGroupOptionsOrientation(string groupOptionsOrientation)
        {
            if (string.IsNullOrEmpty(groupOptionsOrientation)) { throw new System.ArgumentNullException("Parameter groupOptionsOrientation is null."); }
            this.groupOptionsOrientation = groupOptionsOrientation;
            return this;
        }
        public GroupRequest build()
        {
            return new GroupRequest(this);
        }


        [DataContract]
        public class GroupRequest : IJsonSerializable
        {
            [ScriptIgnore]
            private readonly string listId;
            [ScriptIgnore]
            private readonly string groupField;
            [DataMember(Name = "name")]
            private readonly string name;
            [DataMember(Name = "type")]
            private readonly string type;
            [DataMember(Name = "selectionType")]
            private readonly string selectionType;
            [DataMember(Name = "groupOptionsOrientation")]
            private readonly string groupOptionsOrientation;
            [DataMember(Name = "internal")]
            private readonly bool internal_;

            public GroupRequest(GroupRequestBuilder builder)
            {

                this.listId = builder.listId;
                this.groupField = builder.groupField;
                this.name = builder.name;
                this.type = builder.type;
                this.internal_ = builder.internal_;
                this.selectionType = builder.selectionType;
                this.groupOptionsOrientation = builder.groupOptionsOrientation;

            }

            public string getListId()
            {
                return listId;
            }

            public string getGroupField()
            {
                return groupField;
            }

            public string getName()
            {
                return name;
            }

            public string getType()
            {
                return type;
            }

            public string getSelectionType()
            {
                return selectionType;
            }

            public string getGroupOptionsOrientation()
            {
                return groupOptionsOrientation;
            }

            public bool isInternal()
            {
                return internal_;
            }
            public string getBody()
            {
                return JsonConvert.SerializeObject(this, Formatting.Indented,
                            new JsonSerializerSettings
                            {
                                NullValueHandling = NullValueHandling.Ignore
                            });
            }

        }
    }
}
