﻿using apifon_sdk_csharp.com.apifon.sdk.helpers;
using com.apifon.sdk.helpers;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace apifon_sdk_csharp.com.apifon.sdk.model.request
{
    public class ListQueryBuilder : PaginatedRequestBuilder, IFieldSerializable<ListQueryBuilder>
    {
        private ListType type;
        private readonly HashSet<string> fields;

        private ListQueryBuilder()
        {
            this.fields = new HashSet<string>();
        }

        public static ListQueryBuilder getBuilder()
        {
            return new ListQueryBuilder();
        }

        public ListQueryBuilder withType(ListType type)
        {
            if (string.IsNullOrEmpty(type.ToString())) { throw new System.ArgumentNullException("Parameter type is null."); }
            this.type = type;
            return this;
        }

        public override string toQueryParamRequest()
        {
            return new ListQueryRequest(this).toQueryParamRequest();
        }

        public override PaginatedRequest build()
        {
            return new ListQueryRequest(this);
        }

        public HashSet<string> getFields()
        {
            return this.fields;
        }

        public ListQueryBuilder addVisibleFields(Collection<string> fields)
        {
            if (fields == null) { throw new System.ArgumentNullException("Parameter fields is null."); }
            this.getFields().UnionWith(fields);
            return this;
        }

        public ListQueryBuilder addVisibleField(string field)
        {
            if (field == null) { throw new System.ArgumentNullException("Parameter field is null."); }
            this.getFields().Add(field);
            return this;
        }

        public override string getPathId()
        {
            throw new NotImplementedException();
        }

        public string getQueryParams()
        {
            throw new NotImplementedException();
        }

        public class ListQueryRequest : PaginatedRequest
        {

            private readonly ListType type;
            private readonly HashSet<string> fields;

            public ListQueryRequest(ListQueryBuilder builder)
            {
                this.type = builder.type;
                this.fields = builder.fields;
                this.size = builder.size;
                this.page = builder.page;
                this.sortField = builder.sortField;
                this.sortDir = builder.sortDir;
            }

            public override string getPathId()
            {
                throw new NotImplementedException();
            }

            public override string toQueryParamRequest()
            {
                List<KeyValuePair<string, string>> queryParams = new List<KeyValuePair<string, string>>();
                addQueryParams(queryParams);
                if (type.ToString() != null)
                {
                    queryParams.Add(new KeyValuePair<string, string>("type", type.ToString()));
                }
                if (fields != null && fields.Any(x => true))
                {
                    string join = string.Join(",", fields);
                    queryParams.Add(new KeyValuePair<string, string>("fields", join));
                }

                string joinParams = "";
                foreach (KeyValuePair<string, string> entry in queryParams)
                {
                    if (joinParams.Equals(""))
                    {
                        //For the first time
                        joinParams = entry.Key + "=" + entry.Value;
                    }
                    else 
                    {
                        //Add "&" in query
                        joinParams = joinParams + "&" + entry.Key + "=" + entry.Value;
                    }
                }

                UriBuilder uriBuilder = new UriBuilder();
                uriBuilder.Query = joinParams;
                return uriBuilder.Query;
            }

        }
    }
}
