﻿using apifon_sdk_csharp.com.apifon.sdk.helpers;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using System.Web.Script.Serialization;

namespace apifon_sdk_csharp.com.apifon.sdk.model.request
{
    public class SubscriberConditionQueryRequestBuilder : IFieldSerializable<SubscriberConditionQueryRequestBuilder>
    {
        private readonly string listId;
        private int page;
        private int size;
        private readonly List<Condition> conditions;
        private readonly HashSet<string> fields;
        private Sort sort;

        private SubscriberConditionQueryRequestBuilder(String listId)
        {
            this.listId = listId;
            this.page = 1;
            this.size = 200;
            this.conditions = new List<Condition>();
            this.fields = new HashSet<string>();
            this.sort = Sort.getDefaultSort();
        }

        public static SubscriberConditionQueryRequestBuilder getSubscriberConditionsBuilder(string listId)
        {
            return new SubscriberConditionQueryRequestBuilder(listId);
        }

        public SubscriberConditionQueryRequestBuilder clearConditions()
        {
            this.conditions.Clear();
            return this;
        }

        public SubscriberConditionQueryRequestBuilder withCondition(string name, string Operator, Object value)
        {
            this.conditions.Add(new Condition(name, Operator, value));
            return this;
        }

        public SubscriberConditionQueryRequestBuilder withCondition(Condition condition)
        {
            this.conditions.Add(condition);
            return this;
        }

        public SubscriberConditionQueryRequestBuilder withConditions(Collection<Condition> conditions)
        {
            this.conditions.AddRange(conditions);
            return this;
        }

        public SubscriberConditionQueryRequestBuilder withPage(int page)
        {
            this.page = page;
            return this;
        }

        public SubscriberConditionQueryRequestBuilder withSize(int size)
        {
            this.size = size;
            return this;
        }

        public SubscriberConditionQueryRequestBuilder withSort(Sort sort)
        {
            this.sort = sort;
            return this;
        }

        public SubscriberConditionQueryRequestBuilder withSort(string sortDirection, string sortField)
        {
            this.sort = new Sort(sortDirection, sortField);
            return this;
        }

        public SubscriberConditionQueryRequest build()
        {
            return new SubscriberConditionQueryRequest(this);
        }

        public HashSet<string> getFields()
        {
            return this.fields;
        }

        public string getPathId()
        {
            return this.listId;
        }

        public SubscriberConditionQueryRequestBuilder addVisibleFields(Collection<string> fields)
        {
            if (fields == null) { throw new System.ArgumentNullException("Parameter fields is null."); }
            this.getFields().UnionWith(fields);
            return this;
        }

        public SubscriberConditionQueryRequestBuilder addVisibleField(string field)
        {
            if (field == null) { throw new System.ArgumentNullException("Parameter field is null."); }
            this.getFields().Add(field);
            return this;
        }

        public string getQueryParams()
        {
            throw new NotImplementedException();
        }

        [DataContract]
        public class SubscriberConditionQueryRequest : IJsonSerializable
        {
            [ScriptIgnore]
            private readonly string listId;
            [DataMember(Name = "page")]
            private readonly int page;
            [DataMember(Name = "size")]
            private readonly int size;
            [DataMember(Name = "conditions")]
            private readonly List<Condition> conditions;
            [DataMember(Name = "fields")]
            private readonly HashSet<string> fields;
            [DataMember(Name = "sort")]
            private readonly Sort sort;


            public SubscriberConditionQueryRequest(SubscriberConditionQueryRequestBuilder builder)
            {
                this.listId = builder.listId;
                this.page = builder.page;
                this.size = builder.size;
                this.conditions = builder.conditions;
                this.fields = builder.fields;
                this.sort = builder.sort;
            }

            public string getListId()
            {
                return listId;
            }

            public int getPage()
            {
                return page;
            }

            public int getSize()
            {
                return size;
            }

            public List<Condition> getConditions()
            {
                return conditions;
            }

            public HashSet<string> getFields()
            {
                return fields;
            }

            public Sort getSort()
            {
                return sort;
            }

            public string getBody()
            {
                return JsonConvert.SerializeObject(this, Formatting.Indented,
                            new JsonSerializerSettings
                            {
                                NullValueHandling = NullValueHandling.Ignore
                            });
            }
        }
    }
}
