﻿using apifon_sdk_csharp.com.apifon.sdk.exception;
using apifon_sdk_csharp.com.apifon.sdk.resource.helpers;
using apifon_sdk_csharp.com.apifon.sdk.response;
using com.apifon.sdk;
using com.apifon.sdk.resource;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using static apifon_sdk_csharp.com.apifon.sdk.model.request.OAuthConfigurationBuilder;

namespace apifon_sdk_csharp.com.apifon.sdk.resource
{
    public class OAuthResource : AbstractResource
    {
        private static readonly string AUTHORIZE_URL = "oauth2/authorize";
        private static readonly string TOKEN_URL = "oauth2/token";
        private static readonly string REVOKE_URL = "oauth2/revoke";
        private readonly OAuthConfiguration authorizationConfig;
        private readonly OAuthCredentials oauthCredentials;

        public OAuthResource(OAuthConfiguration authorizationConfig) : base(Mookee.GetInstance().getIdentityServerUrl())
        {
            this.authorizationConfig = authorizationConfig;
            this.oauthCredentials = new OAuthCredentials();
        }


        public void getTokensForClientCredentialsGrant()
        {
            if (!this.getGrantType().Equals(OAuthGrantType.CLIENT_CREDENTIALS.getGrantTypeAsString()))
            {
                throw new OAuthTokenException("Tried to retrieve tokens for client credentials but grant type is not client_credentials");
            }
            try
            {
                string queryUrl = "?grant_type=GRANT_TYPE&client_id=CLIENT_ID&client_secret=CLIENT_SECRET&scope=REQUESTED_SCOPES"
                                .Replace("GRANT_TYPE", this.authorizationConfig.getGrantType())
                                .Replace("CLIENT_ID", this.authorizationConfig.getClientId())
                                .Replace("CLIENT_SECRET", this.authorizationConfig.getClientSecret())
                                .Replace("REQUESTED_SCOPES", this.authorizationConfig.getScopesAsString());
                WebRequest webRequest = WebRequest.Create(base.GetEndpoint() + TOKEN_URL + queryUrl);
                webRequest.Method = "POST";
                webRequest.ContentType = "application/x-www-form-urlencoded";
                
                using (HttpWebResponse response = (HttpWebResponse)webRequest.GetResponse())
                {
                    string responseContent = this.getResponseBody(response);
                    
                    if (response.StatusCode == HttpStatusCode.OK)
                    {
                        OAuthCredentials credentials = JsonConvert.DeserializeObject<OAuthCredentials>(responseContent);
                        oauthCredentials.setAccessCode(null);
                        oauthCredentials.setAccessToken(credentials.getAccessToken());
                        oauthCredentials.setRefreshToken(null);
                        oauthCredentials.setExpiresIn(credentials.getExpiresIn());
                    }  
                }
            }
            catch (WebException ex)
            {
                string responseContent = this.getResponseBody(ex.Response);
                OauthTokenErrorResponse resp = JsonConvert.DeserializeObject<OauthTokenErrorResponse>(responseContent);
                OAuthTokenException oAuthTokenException = new OAuthTokenException(ex.Message,resp);
                Console.WriteLine(oAuthTokenException.ToString());
                throw oAuthTokenException;
            }
            catch (Exception ex)
            {
                throw new OAuthTokenException(ex);
            }
        }

        public void revokeTokens()
        {
            revokeAccessToken();
        }

        private bool revokeAccessToken()
        {
            if (this.oauthCredentials.getAccessToken() != null) 
            {
                revokeToken(this.oauthCredentials.getAccessToken(), "access_token");
                this.oauthCredentials.setAccessToken(null);
                return true;
            }
            return false;
        }

        public void refreshToken()
        {
            if (this.getGrantType() == null)
            {
                return;
            }
            switch (this.getGrantType()) {
                case "client_credentials":
                    getTokensForClientCredentialsGrant();
                    break;
            }
        }

        private string getGrantType()
        {
            return this.authorizationConfig.getGrantType();
        }

        public string getAccessToken()
        {
            return this.oauthCredentials.getAccessToken();
        }

        public string getRefreshToken()
        {
            return this.oauthCredentials.getRefreshToken();
        }

        public void revokeToken(string token, string tokenTypeHint)
        {
            try
            {
                string queryUrl = "?client_id=" + this.authorizationConfig.getClientId() + "&token=" + token + "&token_type_hint=" + tokenTypeHint;

                if (OAuthGrantType.CLIENT_CREDENTIALS.getGrantTypeAsString().Equals(this.authorizationConfig.getGrantType()))
                {
                    queryUrl += "&client_secret=" + this.authorizationConfig.getClientSecret();
                }

                WebRequest webRequest = WebRequest.Create(base.GetEndpoint() + REVOKE_URL + queryUrl);
                webRequest.Method = "POST";
                webRequest.ContentType = "application/x-www-form-urlencoded";

                HttpWebResponse response = (HttpWebResponse)webRequest.GetResponse();
            }
            catch (WebException ex)
            {
                var response = ex.Response as HttpWebResponse;
                string responseContent = this.getResponseBody(response);

                if (response.StatusCode != HttpStatusCode.OK)
                {
                    throw new OAuthTokenException("Failed to revoke token with status " + (int)response.StatusCode
                            + " and response " + responseContent);
                }    
            }
            catch (IOException ex)
            {
                throw new OAuthTokenException("Failed to revoke refresh_token with exception", ex);
            }
            catch (Exception ex)
            {
                throw new OAuthTokenException(ex);
            }
        }

        public OAuthCredentials getOauthCredentials()
        {
            return oauthCredentials;
        }
    }
}
